@@ -62,7 +62,8 @@ typedef struct _SwitchScreen {
     Window            popupWindow;
     CompTimeoutHandle popupDelayHandle;
 
-    Window selectedWindow;
+    CompWindow *selectedWindow;
+
     Window clientLeader;
 
     unsigned int previewWidth;
@@ -90,7 +91,7 @@ typedef struct _SwitchScreen {
     unsigned int fgColor[4];
 } SwitchScreen;
 
-#define ICON_SIZE 64
+#define ICON_SIZE 48
 #define MAX_ICON_SIZE 256
 
 #define PREVIEWSIZE 150
@@ -102,12 +103,17 @@ typedef struct _SwitchScreen {
 static void
 setSelectedWindowHint (CompScreen *s)
 {
+    Window selectedWindowId = None;
+
     SWITCH_DISPLAY (s->display);
     SWITCH_SCREEN (s);
 
+    if (ss->selectedWindow && !ss->selectedWindow->destroyed)
+	selectedWindowId = ss->selectedWindow->id;
+
     XChangeProperty (s->display->display, ss->popupWindow, sd->selectWinAtom,
 		     XA_WINDOW, 32, PropModeReplace,
-		     (unsigned char *) &ss->selectedWindow, 1);
+		     (unsigned char *) &selectedWindowId, 1);
 }
 
 static Bool
@@ -117,6 +123,9 @@ isSwitchWin (CompWindow *w)
 
     SWITCH_SCREEN (s);
 
+    if (w->destroyed)
+	return FALSE;
+
     if (!w->mapNum || w->attrib.map_state != IsViewable)
     {
 	if (staticswitcherGetMinimized (s))
@@ -193,7 +202,8 @@ switchActivateEvent (CompScreen *s,
     o[1].name = "active";
     o[1].value.b = activating;
 
-    (*s->display->handleCompizEvent) (s->display, "switcher", "activate", o, 2);
+    (*s->display->handleCompizEvent) (s->display, "staticswitcher", "activate",
+    				      o, 2);
 }
 
 static int
@@ -311,7 +321,7 @@ switchUpdateWindowList (CompScreen *s,
     ss->pos  = 0.0;
     ss->move = 0.0;
 
-    ss->selectedWindow = ss->windows[0]->id;
+    ss->selectedWindow = ss->windows[0];
 
     if (ss->popupWindow)
 	switchUpdatePopupWindow (s, count);
@@ -418,7 +428,7 @@ switchToWindow (CompScreen *s,
 
     for (cur = 0; cur < ss->nWindows; cur++)
     {
-	if (ss->windows[cur]->id == ss->selectedWindow)
+	if (ss->windows[cur] == ss->selectedWindow)
 	    break;
     }
 
@@ -434,7 +444,7 @@ switchToWindow (CompScreen *s,
 
     if (w)
     {
-	Window old = ss->selectedWindow;
+	CompWindow *old = ss->selectedWindow;
 
 	if (ss->selection == AllViewports && staticswitcherGetAutoChangeVp (s))
 	{
@@ -461,9 +471,9 @@ switchToWindow (CompScreen *s,
 			&xev);
 	}
 
-	ss->selectedWindow = w->id;
+	ss->selectedWindow = w;
 
-	if (old != w->id)
+	if (old != w)
 	{
 	    ss->move = nextIdx;
 
@@ -483,12 +493,8 @@ switchToWindow (CompScreen *s,
 
 	switchDoWindowDamage (w);
 
-	if (old)
-	{
-	    w = findWindowAtScreen (s, old);
-	    if (w)
-		switchDoWindowDamage (w);
-	}
+	if (old && !old->destroyed)
+	    switchDoWindowDamage (old);
     }
 }
 
@@ -590,11 +596,11 @@ switchInitiate (CompScreen            *s,
 
     SWITCH_SCREEN (s);
 
-    if (otherScreenGrabExist (s, "switcher", "scale", "cube", 0))
+    if (otherScreenGrabExist (s, "cube", NULL))
 	return;
 
     ss->selection      = selection;
-    ss->selectedWindow = None;
+    ss->selectedWindow = NULL;
 
     count = switchCountWindows (s);
     if (count < 1)
@@ -749,19 +755,15 @@ switchTerminate (CompDisplay     *d,
 	    ss->switching = FALSE;
 
 	    if (state & CompActionStateCancel)
-		ss->selectedWindow = None;
+		ss->selectedWindow = NULL;
 
-	    if (state && ss->selectedWindow)
-	    {
-		w = findWindowAtScreen (s, ss->selectedWindow);
-		if (w)
-		    sendWindowActivationRequest (w->screen, w->id);
-	    }
+	    if (state && ss->selectedWindow && !ss->selectedWindow->destroyed)
+		sendWindowActivationRequest (s, ss->selectedWindow->id);
 
 	    removeScreenGrab (s, ss->grabIndex, 0);
 	    ss->grabIndex = 0;
 
-	    ss->selectedWindow = None;
+	    ss->selectedWindow = NULL;
 
 	    switchActivateEvent (s, FALSE);
 	    setSelectedWindowHint (s);
@@ -941,16 +943,14 @@ switchPrevPanel (CompDisplay     *d,
 
 static void
 switchWindowRemove (CompDisplay *d,
-		    Window	id)
+		    CompWindow  *w)
 {
-    CompWindow *w;
-
-    w = findWindowAtDisplay (d, id);
     if (w)
     {
 	Bool   inList = FALSE;
 	int    count, j, i = 0;
-	Window selected, old;
+	CompWindow *selected;
+	CompWindow *old;
 	CompScreen *s = w->screen;
 
 	SWITCH_SCREEN (s);
@@ -966,12 +966,12 @@ switchWindowRemove (CompDisplay *d,
 	    {
 		inList = TRUE;
 
-		if (w->id == selected)
+		if (w == selected)
 		{
-		    if (i < ss->nWindows)
-			selected = ss->windows[i + 1]->id;
+		    if (i + 1 < ss->nWindows)
+			selected = ss->windows[i + 1];
 		    else
-			selected = ss->windows[0]->id;
+			selected = ss->windows[0];
 		}
 
 		ss->nWindows--;
@@ -1008,7 +1008,7 @@ switchWindowRemove (CompDisplay *d,
 
 	for (i = 0; i < ss->nWindows; i++)
 	{
-	    ss->selectedWindow = ss->windows[i]->id;
+	    ss->selectedWindow = ss->windows[i];
 	    ss->move = ss->pos = i;
 
 	    if (ss->selectedWindow == selected)
@@ -1028,11 +1028,11 @@ switchWindowRemove (CompDisplay *d,
 
 	if (old != ss->selectedWindow)
 	{
+	    switchDoWindowDamage (ss->selectedWindow);
 	    switchDoWindowDamage (w);
 
-	    w = findWindowAtScreen (w->screen, old);
-	    if (w)
-		switchDoWindowDamage (w);
+	    if (old && !old->destroyed)
+		switchDoWindowDamage (old);
 
 	    ss->moreAdjust = 1;
 	}
@@ -1132,7 +1132,7 @@ switchGetWindowPosition (CompScreen   *s,
     *y = row * ss->previewHeight + (row + 1) * ss->previewBorder;
 }
 
-static Window
+static CompWindow *
 switchFindWindowAt (CompScreen *s,
 		    int        x,
 		    int        y)
@@ -1159,11 +1159,11 @@ switchFindWindowAt (CompScreen *s,
 	    y2 = y1 + ss->previewHeight;
 
 	    if (x >= x1 && x < x2 && y >= y1 && y < y2)
-		return ss->windows[i]->id;
+		return ss->windows[i];
 	}
     }
 
-    return None;
+    return NULL;
 }
 
 static void
@@ -1171,7 +1171,7 @@ switchHandleEvent (CompDisplay *d,
 		   XEvent      *event)
 {
     CompScreen *s;
-    CompWindow *w;
+    CompWindow *w = NULL;
 
     SWITCH_DISPLAY (d);
 
@@ -1184,6 +1184,9 @@ switchHandleEvent (CompDisplay *d,
 
 	    if (w->id == ss->popupWindow)
 	    {
+		/* we don't get a MapRequest for internal window creations,
+		   so we need to set the internals ourselves */
+		w->managed = TRUE;
 		w->wmType = getWindowType (d, w->id);
 		recalcWindowType (w);
 		recalcWindowActions (w);
@@ -1191,6 +1194,13 @@ switchHandleEvent (CompDisplay *d,
 	    }
 	}
 	break;
+    case DestroyNotify:
+	/* We need to get the CompWindow * for event->xdestroywindow.window
+	   here because in the (*d->handleEvent) call below, that CompWindow's
+	   id will become 1, so findWindowAtDisplay won't be able to find the
+	   CompWindow after that. */
+	w = findWindowAtDisplay (d, event->xdestroywindow.window);
+	break;
     }
 
     UNWRAP (sd, d, handleEvent);
@@ -1199,10 +1209,11 @@ switchHandleEvent (CompDisplay *d,
 
     switch (event->type) {
     case UnmapNotify:
-	switchWindowRemove (d, event->xunmap.window);
+	w = findWindowAtDisplay (d, event->xunmap.window);
+	switchWindowRemove (d, w);
 	break;
     case DestroyNotify:
-	switchWindowRemove (d, event->xdestroywindow.window);
+	switchWindowRemove (d, w);
 	break;
     case PropertyNotify:
 	if (event->xproperty.atom == sd->selectFgColorAtom)
@@ -1225,7 +1236,7 @@ switchHandleEvent (CompDisplay *d,
 
 	    if (ss->grabIndex && ss->mouseSelect)
 	    {
-		Window selected;
+		CompWindow *selected;
 
 		selected = switchFindWindowAt (s,
 					       event->xbutton.x_root,
@@ -1366,12 +1377,13 @@ switchPaintOutput (CompScreen		   *s,
 
 	if (mode == HighlightModeBringSelectedToFront)
 	{
-	    zoomed = findWindowAtScreen (s, ss->selectedWindow);
-	    if (zoomed)
+	    zoomed = ss->selectedWindow;
+	    if (zoomed && !zoomed->destroyed)
 	    {
 		CompWindow *w;
 
-		for (w = zoomed->prev; w && w->id <= 1; w = w->prev);
+		for (w = zoomed->prev; w && w->id <= 1; w = w->prev)
+		    ;
 		zoomedAbove = (w) ? w->id : None;
 
 		unhookWindowFromScreen (s, zoomed);
@@ -1410,7 +1422,7 @@ switchPaintOutput (CompScreen		   *s,
 		if (zoomed)
 		    w = zoomed;
 		else
-		    w = findWindowAtScreen (s, ss->selectedWindow);
+		    w = ss->selectedWindow;
 
 		if (w)
 		{
@@ -1527,6 +1539,7 @@ switchPaintThumb (CompWindow		  *w,
 	FragmentAttrib	      fragment;
 	CompTransform	      wTransform = *transform;
 	int		      ww, wh;
+	GLenum                filter;
 
 	width  = ss->previewWidth;
 	height = ss->previewHeight;
@@ -1556,7 +1569,7 @@ switchPaintThumb (CompWindow		  *w,
 	wy = y + (ss->previewHeight / 2) - (height / 2);
 
 #if 0
-	if (w->id != ss->selectedWindow)
+	if (w != ss->selectedWindow)
 	{
 	    sAttrib.brightness /= 2;
 	    sAttrib.saturation /= 2;
@@ -1581,6 +1594,11 @@ switchPaintThumb (CompWindow		  *w,
 	glPushMatrix ();
 	glLoadMatrixf (wTransform.m);
 
+	filter = s->display->textureFilter;
+
+	if (staticswitcherGetMipmap (s))
+	    s->display->textureFilter = GL_LINEAR_MIPMAP_LINEAR;
+
 	/* XXX: replacing the addWindowGeometry function like this is
 	   very ugly but necessary until the vertex stage has been made
 	   fully pluggable. */
@@ -1590,6 +1608,8 @@ switchPaintThumb (CompWindow		  *w,
 				 mask);
 	w->screen->addWindowGeometry = oldAddWindowGeometry;
 
+	s->display->textureFilter = filter;
+
 	glPopMatrix ();
 
 	if (staticswitcherGetIcon (s))
@@ -1599,8 +1619,8 @@ switchPaintThumb (CompWindow		  *w,
 	    {
 		float xScale, yScale;
 
-		xScale = (icon->width > ICON_SIZE) ? (float) ICON_SIZE / icon->width : 1.0;
-		yScale = (icon->height > ICON_SIZE) ? (float) ICON_SIZE / icon->height : 1.0;
+		xScale = (float) ICON_SIZE / icon->width;
+		yScale = (float) ICON_SIZE / icon->height;
 
 		if (xScale < yScale)
 		    yScale = xScale;
@@ -1620,30 +1640,15 @@ switchPaintThumb (CompWindow		  *w,
 	width  = ss->previewWidth * 3 / 4;
 	height = ss->previewHeight * 3 / 4;
 
-	/* try to get a matching icon first */
-	icon = getWindowIcon (w, width, height);
-	/* if none found, try a large one */
-	if (!icon)
-	    icon = getWindowIcon (w, MAX_ICON_SIZE, MAX_ICON_SIZE);
+	/* try to get the largest icon */
+	icon = getWindowIcon (w, MAX_ICON_SIZE, MAX_ICON_SIZE);
 	if (!icon)
 	    icon = w->screen->defaultIcon;
 
 	if (icon)
 	{
-	    float iw, ih;
-
-	    iw = width;
-	    ih = height;
-
-	    if (icon->width < (iw / 2) || icon->width > iw)
-		sAttrib.xScale = (iw / icon->width);
-	    else
-		sAttrib.xScale = 1.0f;
-
-	    if (icon->height < (ih / 2) || icon->height > ih)
-		sAttrib.yScale = (ih / icon->height);
-	    else
-		sAttrib.yScale = 1.0f;
+	    sAttrib.xScale = (width / icon->width);
+	    sAttrib.yScale = (height / icon->height);
 
 	    if (sAttrib.xScale < sAttrib.yScale)
 		sAttrib.yScale = sAttrib.xScale;
@@ -1780,7 +1785,6 @@ switchPaintWindow (CompWindow		   *w,
 
     if (w->id == ss->popupWindow)
     {
-	GLenum         filter;
 	int            x, y, offX, i;
 	float          px, py, pos;
 
@@ -1794,11 +1798,6 @@ switchPaintWindow (CompWindow		   *w,
 	if (!(mask & PAINT_WINDOW_TRANSFORMED_MASK) && region->numRects == 0)
 	    return TRUE;
 
-	filter = s->display->textureFilter;
-
-	if (staticswitcherGetMipmap (s))
-	    s->display->textureFilter = GL_LINEAR_MIPMAP_LINEAR;
-
 	glPushAttrib (GL_SCISSOR_BIT);
 
 	glEnable (GL_SCISSOR_TEST);
@@ -1811,8 +1810,6 @@ switchPaintWindow (CompWindow		   *w,
 			      mask, x + w->attrib.x, y + w->attrib.y);
 	}
 
-	s->display->textureFilter = filter;
-
 	pos = fmod (ss->pos, ss->nWindows);
 	px  = fmod (pos, ss->xCount);
 	py  = floor (pos / ss->xCount);
@@ -1850,7 +1847,7 @@ switchPaintWindow (CompWindow		   *w,
 	glPopAttrib ();
     }
     else if (ss->switching && !ss->popupDelayHandle &&
-	     (w->id != ss->selectedWindow))
+	     (w != ss->selectedWindow))
     {
 	WindowPaintAttrib sAttrib = *attrib;
 	GLuint            value;
@@ -2020,7 +2017,7 @@ switchInitScreen (CompPlugin *p,
     ss->popupWindow      = None;
     ss->popupDelayHandle = 0;
 
-    ss->selectedWindow = None;
+    ss->selectedWindow = NULL;
     ss->clientLeader   = None;
 
     ss->windows     = 0;
