@@ -107,6 +107,8 @@ typedef struct _WallScreen
     PreparePaintScreenProc       preparePaintScreen;
     PaintTransformedOutputProc   paintTransformedOutput;
     PaintWindowProc              paintWindow;
+    WindowGrabNotifyProc         windowGrabNotify;
+    WindowUngrabNotifyProc       windowUngrabNotify;
     ActivateWindowProc           activateWindow;
 
     Bool moving; /* Used to track miniview movement */
@@ -126,6 +128,8 @@ typedef struct _WallScreen
 
     Window moveWindow;
 
+    CompWindow *grabWindow;
+
     Bool focusDefault;
 
     ScreenTransformation transform;
@@ -549,7 +553,7 @@ wallMoveViewport (CompScreen *s,
     if (!x && !y)
 	return FALSE;
 
-    if (otherScreenGrabExist (s, "move", "switcher", "group-drag", "wall", 0))
+    if (otherScreenGrabExist (s, "move", "switcher", "group-drag", "wall", NULL))
 	return FALSE;
 
     if (!wallCheckDestination (s, x, y))
@@ -625,7 +629,7 @@ wallHandleEvent (CompDisplay *d,
 	    if (!s)
 		break;
 
-	    if (otherScreenGrabExist (s, "switcher", "wall", 0))
+	    if (otherScreenGrabExist (s, "switcher", "wall", NULL))
 		break;
 
     	    dx = event->xclient.data.l[0] / s->width - s->x;
@@ -651,7 +655,7 @@ wallActivateWindow (CompWindow *w)
 
     WALL_SCREEN (s);
 
-    if (w->placed && !otherScreenGrabExist (s, "wall", "switcher", 0))
+    if (w->placed && !otherScreenGrabExist (s, "wall", "switcher", NULL))
     {
 	int dx, dy;
 
@@ -756,7 +760,7 @@ wallInitiateFlip (CompScreen *s,
     int dx, dy;
     int amountX, amountY;
 
-    if (otherScreenGrabExist (s, "wall", "move", "group-drag", 0))
+    if (otherScreenGrabExist (s, "wall", "move", "group-drag", NULL))
 	return FALSE;
 
     if (dnd)
@@ -764,16 +768,25 @@ wallInitiateFlip (CompScreen *s,
 	if (!wallGetEdgeflipDnd (s))
 	    return FALSE;
 
-	if (otherScreenGrabExist (s, "wall", 0))
+	if (otherScreenGrabExist (s, "wall", NULL))
 	    return FALSE;
     }
-    else if (otherScreenGrabExist (s, "wall", "group-drag", 0))
+    else if (otherScreenGrabExist (s, "wall", "group-drag", NULL))
     {
 	/* not wall or group means move */
 	if (!wallGetEdgeflipMove (s))
 	    return FALSE;
+
+	WALL_SCREEN (s);
+
+	if (!ws->grabWindow)
+	    return FALSE;
+
+	/* bail out if window is sticky */
+	if (ws->grabWindow->state & CompWindowStateStickyMask)
+	    return FALSE;
     }
-    else if (otherScreenGrabExist (s, "wall", 0))
+    else if (otherScreenGrabExist (s, "wall", NULL))
     {
 	/* move was ruled out before, so we have group */
 	if (!wallGetEdgeflipDnd (s))
@@ -1883,6 +1896,36 @@ wallMatchPropertyChanged (CompDisplay *d,
     ww->isSliding = !matchEval (wallGetNoSlideMatch (d), w);
 }
 
+static void
+wallWindowGrabNotify (CompWindow   *w,
+		      int	     x,
+		      int	     y,
+		      unsigned int state,
+		      unsigned int mask)
+{
+    WALL_SCREEN (w->screen);
+
+    if (!ws->grabWindow)
+	ws->grabWindow = w;
+
+    UNWRAP (ws, w->screen, windowGrabNotify);
+    (*w->screen->windowGrabNotify) (w, x, y, state, mask);
+    WRAP (ws, w->screen, windowGrabNotify, wallWindowGrabNotify);
+}
+
+static void
+wallWindowUngrabNotify (CompWindow *w)
+{
+    WALL_SCREEN (w->screen);
+
+    if (w == ws->grabWindow)
+	ws->grabWindow = NULL;
+
+    UNWRAP (ws, w->screen, windowUngrabNotify);
+    (*w->screen->windowUngrabNotify) (w);
+    WRAP (ws, w->screen, windowUngrabNotify, wallWindowUngrabNotify);
+}
+
 static void
 wallWindowAdd (CompScreen *s,
 	       CompWindow *w)
@@ -2074,6 +2117,7 @@ wallInitScreen (CompPlugin *p,
     ws->showPreview  = FALSE;
     ws->focusDefault = TRUE;
     ws->moveWindow   = None;
+    ws->grabWindow   = NULL;
 
     ws->transform  = NoTransformation;
     ws->direction  = -1;
@@ -2089,6 +2133,8 @@ wallInitScreen (CompPlugin *p,
     WRAP (ws, s, paintTransformedOutput, wallPaintTransformedOutput);
     WRAP (ws, s, preparePaintScreen, wallPreparePaintScreen);
     WRAP (ws, s, paintWindow, wallPaintWindow);
+    WRAP (ws, s, windowGrabNotify, wallWindowGrabNotify);
+    WRAP (ws, s, windowUngrabNotify, wallWindowUngrabNotify);
     WRAP (ws, s, activateWindow, wallActivateWindow);
 
     s->base.privates[wd->screenPrivateIndex].ptr = ws;
@@ -2118,6 +2164,8 @@ wallFiniScreen (CompPlugin *p,
     UNWRAP (ws, s, paintTransformedOutput);
     UNWRAP (ws, s, preparePaintScreen);
     UNWRAP (ws, s, paintWindow);
+    UNWRAP (ws, s, windowGrabNotify);
+    UNWRAP (ws, s, windowUngrabNotify);
     UNWRAP (ws, s, activateWindow);
     
     freeWindowPrivateIndex (s, ws->windowPrivateIndex);
